<?php
/**
 * Joomla! 1.5 component Community Polls
 *
 * @version $Id: helper.php 2009-08-10 03:45:15 svn $
 * @author Maverick
 * @package Joomla
 * @subpackage Community Polls
 * @license GNU/GPL
 *
 * The Community Polls allows the members of the Joomla website to create and manage polls from the front-end.
 * The administrator has the powerful tools provided in the back-end to manage the polls published by all users.
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Community Polls Helper
 *
 * @package Joomla
 * @subpackage Community Polls
 * @since 1.5
 */
class CommunityPollsHelper {
    function checkModules($position) {
        $db =& JFactory::getDBO();
        $query = 'SELECT COUNT(*) FROM #__modules WHERE position=' .
            $db->quote($position) . ' AND published=1 and access=0';
        $db->setQuery($query);
        $count = $db->loadResult();
        return $count;
    }

    function &populateJcpConfig($forceCheckDB=true) {
        $app = &JFactory::getApplication();
        require_once JPATH_ADMINISTRATOR.DS.'components'.DS.'com_communitypolls'.DS.'helpers'.DS.'constants.php';
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );

        if(!isset($jcp_config) || $forceCheckDB) {
            $db     =& JFactory::getDBO();
            $query = 'SELECT config_name, config_value FROM '. TABLE_JCP_CONFIG;
            $db->setQuery($query);
            $configt = $db->loadObjectList();

            foreach($configt as $ct) {
                $jcp_config[$ct->config_name] = $ct->config_value;
            }
            $app->setUserState( SESSION_JCP_CONFIG, $jcp_config );
        }
        return $jcp_config;
    }
    
    function sendEmail($sender_email, $sender_name, $recievers, $subject, $body ){
        foreach ($recievers as $email){
            if(!in_array($email, $sent)) {
                $mail =& JFactory::getMailer();
                $sender = array( $sender_email, $sender_name );
                $mail->setSender($sender);
                $mail->addRecipient($email);
                $mail->setSubject($subject);
                $mail->setBody($body);
                $mail->send();
                $sent[] = $email;
            }
        }
    }

    function escape($var){
        return htmlspecialchars($var, ENT_COMPAT, 'UTF-8');
    }

    /**
     * Gets the version information of Community Polls.
     * Credits: Bernard Gilly, www.alphaplug.com
     * @return <type> Array
     */
    function checkCPUpdate() {
        $url = 'http://www.corejoomla.com/version.xml';
        $data = '';
        $check = array();
        $check['connect'] = 0;
        $check['current_version'] = COMMUNITY_POLLS_CURR_VERSION;

        //try to connect via cURL
        if(function_exists('curl_init') && function_exists('curl_exec')) {
            $ch = @curl_init();

            @curl_setopt($ch, CURLOPT_URL, $url);
            @curl_setopt($ch, CURLOPT_HEADER, 0);
            //http code is greater than or equal to 300 ->fail
            @curl_setopt($ch, CURLOPT_FAILONERROR, 1);
            @curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            //timeout of 5s just in case
            @curl_setopt($ch, CURLOPT_TIMEOUT, 5);

            $data = @curl_exec($ch);

            @curl_close($ch);
        }

        //try to connect via fsockopen
        if(function_exists('fsockopen') && $data == '') {

            $errno = 0;
            $errstr = '';

            //timeout handling: 5s for the socket and 5s for the stream = 10s
            $fsock = @fsockopen("www.corejoomla.com", 80, $errno, $errstr, 5);

            if ($fsock) {
                @fputs($fsock, "GET /version.xml HTTP/1.1\r\n");
                @fputs($fsock, "HOST: www.corejoomla.com\r\n");
                @fputs($fsock, "Connection: close\r\n\r\n");

                //force stream timeout...
                @stream_set_blocking($fsock, 1);
                @stream_set_timeout($fsock, 5);

                $get_info = false;
                while (!@feof($fsock)) {
                    if ($get_info) {
                        $data .= @fread($fsock, 1024);
                    }
                    else {
                        if (@fgets($fsock, 1024) == "\r\n") {
                            $get_info = true;
                        }
                    }
                }
                @fclose($fsock);

                //need to check data cause http error codes aren't supported here
                if(!strstr($data, '<?xml version="1.0" encoding="utf-8" ?><update>')) {
                    $data = '';
                }
            }
        }

        //try to connect via fopen
        if (function_exists('fopen') && ini_get('allow_url_fopen') && $data == '') {

            //set socket timeout
            ini_set('default_socket_timeout', 5);

            $handle = @fopen ($url, 'r');

            //set stream timeout
            @stream_set_blocking($handle, 1);
            @stream_set_timeout($handle, 5);

            $data	= @fread($handle, 1000);

            @fclose($handle);
        }

        if( $data && strstr($data, '<?xml version="1.0" encoding="utf-8" ?><update>') ) {
            $xml = & JFactory::getXMLparser('Simple');
            $xml->loadString($data);
            $version 			= & $xml->document->version[0];
            $check['version'] 		= & $version->data();
            $released 			= & $xml->document->released[0];
            $check['released'] 		= & $released->data();
            $check['status'] 		= version_compare( $check['current_version'], $check['version'] );
            $check['connect'] 		= 1;
        }

        return $check;
    }

    /**
     * Does the third party activities needed after poll creation/vote casting.
     * Currently supporting Alpha User Points points submission,
     * JomSocial Points submission, JomSocial activity stream and
     * Twitter updates using shortened urls from bit.ly and tinyurl.com services.
     *
     * @global <type> $jcp_config
     * @param <type> $action
     * @param <type> $poll
     */
    function do3rdPartyTask($action, $poll) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
//        $menu = &JSite::getMenu();
//        $mnuitems	= $menu->getItems('link', 'index.php?option=com_communitypolls&controller=polls');
//        $itemid = isset($mnuitems[0]) ? '&amp;Itemid='.$mnuitems[0]->id : '';

        switch ($action) {
            case "VOTE":
            //Alpha User Points Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM],POINTS_SYSTEM_AUP) == 0) {
                    $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                    if ( file_exists($api_AUP)) {
                        require_once ($api_AUP);
                        $aupid = AlphaUserPointsHelper::getAnyUserReferreID( $poll->created_by );
                        if($aupid) {
                            AlphaUserPointsHelper::newpoints( POINTS_SYSTEM_AUP_PLUGIN_VOTES, $aupid,
                                    '', JText::_('CAST_VOTE_AUP') . ' ' . $poll->id, $jcp_config[VOTES_POINTS], true);
                        }
                    }
                }
                //End AUP

                //JomSocial Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM], POINTS_SYSTEM_JOMSOCIAL) == 0) {
                    include_once( JPATH_SITE . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'userpoints.php');
                    CuserPoints::assignPoint(POINTS_SYSTEM_JS_PLUGIN_VOTES, $poll->created_by);
                }
                if($jcp_config[JS_VOTE_ACTIVITY_ENABLED] == '1') {
                    $API = JPATH_SITE . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'userpoints.php';
                    if(file_exists($API)) {
                        include_once( $API );
                        $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                        $act = new stdClass();
                        $act->cmd       = 'wall.write';
                        $act->actor 	= $poll->created_by;
                        $act->target 	= 0; // no target
                        $VOTE_TEXT = JText::_('USER_VOTED_JS_ACTIVITY_TEXT');
                        $act->title 	= JText::_('{actor} ' . $VOTE_TEXT . ' <a href="'.$link.'">'.$poll->title.'</a>');
                        $act->content 	= '';
                        $act->app       = 'wall';
                        $act->cid       = 0;

                        CFactory::load('libraries', 'activities');
                        CActivityStream::add($act);
                    }
                }
                //End JomSocial

                // Mighty Touch Support
                if($jcp_config[JS_VOTE_ACTIVITY_ENABLED] == "1") {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
                    if(file_exists($API)) {
                        require_once $API;
                        $capi = new JSCommunityApi();

                        if($poll->created_by) {
                            $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                    $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                            $text = JText::_('USER_VOTED_JS_ACTIVITY_TEXT') . ' <a href="'.$link.'">' . $poll->title . '</a>';
                            $icon = JURI::base() . 'components/com_communitypolls/assets/images/poll_add.png';
                            $capi->registerActivity(0, $text, $poll->created_by, $icon, 'user', null, 'com_communitypolls', '', 'Polls');
                        }
                    }
                }
                // End Mighty Touch Support
                break;
            case "POLL":
            //Alpha User Points Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM],POINTS_SYSTEM_AUP) == 0) {
                    $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                    if ( file_exists($api_AUP)) {
                        require_once ($api_AUP);
                        $aupid = AlphaUserPointsHelper::getAnyUserReferreID( $poll->created_by );
                        if($aupid) {
                            AlphaUserPointsHelper::newpoints( POINTS_SYSTEM_AUP_PLUGIN_POLLS, $aupid,
                                    '', 'Poll: ' . $poll->id, $jcp_config[POLL_POINTS], true);
                        }
                    }
                }
                //End AUP

                //JomSocial Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM], POINTS_SYSTEM_JOMSOCIAL) == 0) {
                    include_once( JPATH_SITE . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'userpoints.php');
                    CuserPoints::assignPoint(POINTS_SYSTEM_JS_PLUGIN_POLLS, $poll->created_by);
                }

                if($jcp_config[JS_POLL_ACTIVITY_ENABLED] == "1") {
                    $API = JPATH_SITE . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'userpoints.php';
                    if(file_exists($API)) {
                        include_once( $API );
                        $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                        $act = new stdClass();
                        $act->cmd    = 'wall.write';
                        $act->actor  = $poll->created_by;
                        $act->target = 0; // no target
                        $POLL_TEXT = JText::_('POLL_CREATED_JS_ACTIVITY_TEXT');
                        $act->title  = JText::_('{actor} '. $POLL_TEXT .' <a href="'.$link.'">' . $poll->title . '</a>');
                        $act->app    = 'wall';
                        $act->cid    = 0;

                        CFactory::load('libraries', 'activities');
                        CActivityStream::add($act);
                    }
                }
                //End JomSocial

                // Mighty Touch Support
                if($jcp_config[JS_POLL_ACTIVITY_ENABLED] == "1") {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
                    if(file_exists($API)) {
                        require_once $API;
                        $capi = new JSCommunityApi();

                        if($poll->created_by) {
                            $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                    $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                            $text = JText::_('POLL_CREATED_JS_ACTIVITY_TEXT') . ' <a href="'.$link.'">' . $poll->title . '</a>';
                            $icon = JURI::base() . 'components/com_communitypolls/assets/images/poll_add.png';
                            $capi->registerActivity(0, $text, $poll->created_by, $icon, 'user', null, 'com_communitypolls', '', 'Polls');
                        }
                    }
                }
                // End Mighty Touch Support

                // Twitter Tweets Start
                if($jcp_config[ENABLE_TWITTER] == "1") {
                    PollsHelper::sendToTwitter($poll);
                }
                // Twitter Tweets End
                break;
        }
    }

    function sendToTwitter($poll) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $twitter_lib = JPATH_ROOT . DS . 'components' . DS . 'com_communitypolls' . DS . 'helpers' . DS . 'twitter.php';
        echo $twitter_lib;
        if(file_exists($twitter_lib)) {
            require_once($twitter_lib);
            try{
                $twi_user = new Twitter($jcp_config[TWITTER_USER_NAME], $jcp_config[TWITTER_PASSWORD]);
                $user =& JFactory::getUser();
                $poll_url =  JURI::root() . 'index.php?option=com_communitypolls&controller=polls&task=viewpoll&id='. $poll->id . ($poll->alias?(':' . $poll->alias):'');
                $poll_url = PollsHelper::getShortenedUrl($poll_url);
                $twi_text = $user->$jcp_config[USER_NAME] . ' ' . JText::_('TWEET_POLL') . ' ' . $poll_url . ' - ' . $poll->title;
                $twi_text = substr($twi_text, 0, 120);
                $twi_user->updateStatus($twi_text);
            }catch (TwitterException $e){}
        }
    }

    function getShortenedUrl($url) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $shortUrl;
        switch($jcp_config[SHORT_URL_SERVICE]) {
            case 'none':
                return $url;
            case 'bit.ly':
                require_once JPATH_ROOT . DS . 'components' . DS . 'com_communitypolls' . DS . 'helpers' . DS . 'bitly.php';
                $bitly = new Bitly($jcp_config[BIT_LY_LOGIN], $jcp_config[BIT_LY_API_KEY]);
                try {
                    $shortUrl = $bitly->shorten($url);
                }catch (BitlyException $e ) {
                    return false;
                }
                break;
            case 'tinyurl':
                require_once JPATH_ROOT . DS . 'components' . DS . 'com_communitypolls' . DS . 'helpers' . DS . 'tinyurl.php';
                $tinyUrl = new TinyUrl();
                $shortUrl = $tinyUrl->create($url);
                break;
            default:
                $shortUrl = $url;
                break;
        }
        return $shortUrl;
    }
}
?>