<?php
/**
 * Joomla! 1.5 component Community Polls
 *
 * @version $Id: helper.php 2009-08-10 03:45:15 svn $
 * @author Maverick
 * @package Joomla
 * @subpackage Community Polls
 * @license GNU/GPL
 *
 * The Community Polls allows the members of the Joomla website to create and manage polls from the front-end.
 * The administrator has the powerful tools provided in the back-end to manage the polls published by all users.
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

class PollsHelper {
    /**
     * This method will populates the bar chart data such as percent of votes in the poll object passed.
     * @global <type> $jcp_config
     * @param <type> $poll
     */
    function getChartData(&$poll) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $graphwidth     = $jcp_config[CHART_WIDTH];
        $barheight 	= 4;
        $maxcolors 	= 5;
        $barcolor 	= 0;
        $tabcnt 	= 0;
        $colorx 	= 0;

        $total_votes= $poll->votes;

        for ($i = 0; $i < count( $poll->options ); $i++) {
            $vote =& $poll->options[$i];

            if ($total_votes > 0) {
                $vote->percent  = round( 100 * $vote->votes / $total_votes, 1 );
            }else {
                $vote->percent	= 0;
            }
        }
    }

    /**
     * Loads the modules published in the position name passed.
     *
     * @param <type> $position
     * @return <type>
     */
    function loadModulePosition($position) {
        if(JDocumentHTML::countModules($position)) {
            $document	= &JFactory::getDocument();
            $renderer	= $document->loadRenderer('modules');
            $options	= array('style' => 'xhtml');
            return $renderer->render($position, $options, null);
        }else {
            return '';
        }
    }

    /**
     * Does the third party activities needed after poll creation/vote casting.
     * Currently supporting Alpha User Points points submission,
     * JomSocial Points submission, JomSocial activity stream and
     * Twitter updates using shortened urls from bit.ly and tinyurl.com services.
     *
     * @global <type> $jcp_config
     * @param <type> $action
     * @param <type> $poll
     */
    function do3rdPartyTask($action, $poll, $user=null) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        if(null == $user){
            $user =& JFactory::getUser();
        }

        switch ($action) {
            case "VOTE":
            //Alpha User Points Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM],POINTS_SYSTEM_AUP) == 0) {
                    $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                    if ( file_exists($api_AUP)) {
                        require_once ($api_AUP);
                        $aupid = AlphaUserPointsHelper::getAnyUserReferreID( $user->id );
                        if($aupid) {
                            AlphaUserPointsHelper::newpoints( POINTS_SYSTEM_AUP_PLUGIN_VOTES, $aupid,
                                    '', JText::_('CAST_VOTE_AUP') . ' ' . $poll->id, $jcp_config[VOTES_POINTS], true);
                        }
                    }
                }
                //End AUP

                //JomSocial Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM], POINTS_SYSTEM_JOMSOCIAL) == 0) {
                    include_once( JPATH_SITE . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'userpoints.php');
                    CuserPoints::assignPoint(POINTS_SYSTEM_JS_PLUGIN_VOTES, $user->id);
                }
                if($jcp_config[JS_VOTE_ACTIVITY_ENABLED] == '1') {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php';
                    if(file_exists($API)) {
                        include_once( $API );
                        $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                        $act = new stdClass();
                        $act->cmd       = 'wall.write';
                        $act->actor 	= $user->id;
                        $act->target 	= 0; // no target
                        $VOTE_TEXT = JText::_('USER_VOTED_JS_ACTIVITY_TEXT');
                        $act->title 	= JText::_('{actor} ' . $VOTE_TEXT . ' <a href="'.$link.'">'.$poll->title.'</a>');
                        $act->content 	= '';
                        $act->app       = 'wall';
                        $act->cid       = 0;

                        CFactory::load('libraries', 'activities');
                        CActivityStream::add($act);
                    }
                }
                //End JomSocial

                // Mighty Touch Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM], POINTS_SYSTEM_TOUCH) == 0) {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
                    if(file_exists($API)){
                        require_once $API;
                        JSCommunityApi::increaseKarma($user->id, $jcp_config[VOTES_POINTS]);
                    }
                }
                if($jcp_config[JS_VOTE_ACTIVITY_ENABLED] == "1") {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
                    if(file_exists($API)) {
                        require_once $API;
                        $capi = new JSCommunityApi();

                        if($user->id) {
                            $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                    $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                            $text = JText::_('USER_VOTED_JS_ACTIVITY_TEXT') . ' <a href="'.$link.'">' . $poll->title . '</a>';
                            $icon = 'components/com_communitypolls/assets/images/poll_add.png';
                            $capi->registerActivity(0, $text, $user->id, $icon, 'user', null, 'com_communitypolls', '', 'Polls');
                        }
                    }
                }
                // End Mighty Touch Support
                break;
            case "POLL":
            //Alpha User Points Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM],POINTS_SYSTEM_AUP) == 0) {
                    $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                    if ( file_exists($api_AUP)) {
                        require_once ($api_AUP);
                        $aupid = AlphaUserPointsHelper::getAnyUserReferreID( $user->id );
                        if($aupid) {
                            AlphaUserPointsHelper::newpoints( POINTS_SYSTEM_AUP_PLUGIN_POLLS, $aupid,
                                    '', 'Poll: ' . $poll->id, $jcp_config[POLL_POINTS], true);
                        }
                    }
                }
                //End AUP

                //JomSocial Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM], POINTS_SYSTEM_JOMSOCIAL) == 0) {
                    include_once( JPATH_SITE . DS . 'components' . DS . 'com_community' . DS . 'libraries' . DS . 'userpoints.php');
                    CuserPoints::assignPoint(POINTS_SYSTEM_JS_PLUGIN_POLLS, $user->id);
                }

                if($jcp_config[JS_POLL_ACTIVITY_ENABLED] == "1") {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php';
                    if(file_exists($API)) {
                        include_once( $API );
                        $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                        $act = new stdClass();
                        $act->cmd    = 'wall.write';
                        $act->actor  = $user->id;
                        $act->target = 0; // no target
                        $POLL_TEXT = JText::_('POLL_CREATED_JS_ACTIVITY_TEXT');
                        $act->title  = JText::_('{actor} '. $POLL_TEXT .' <a href="'.$link.'">' . $poll->title . '</a>');
                        $act->app    = 'wall';
                        $act->cid    = 0;

                        CFactory::load('libraries', 'activities');
                        CActivityStream::add($act);
                    }
                }
                //End JomSocial

                // Mighty Touch Support
                if(strcasecmp($jcp_config[CONFIG_POINTS_SYSTEM], POINTS_SYSTEM_TOUCH) == 0) {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
                    if(file_exists($API)){
                        require_once $API;
                        JSCommunityApi::increaseKarma($user->id, $jcp_config[POLL_POINTS]);
                    }
                }
                if($jcp_config[JS_POLL_ACTIVITY_ENABLED] == "1") {
                    $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
                    if(file_exists($API)) {
                        require_once $API;
                        $capi = new JSCommunityApi();

                        if($user->id) {
                            $link = JRoute::_('index.php?option=com_communitypolls&amp;controller=polls&amp;task=viewpoll&amp;id='.
                                    $poll->id . ":" . $poll->alias.'&Itemid='.JRequest::getInt('Itemid'));
                            $text = JText::_('POLL_CREATED_JS_ACTIVITY_TEXT') . ' <a href="'.$link.'">' . $poll->title . '</a>';
                            $icon = 'components/com_communitypolls/assets/images/poll_add.png';
                            $capi->registerActivity(0, $text, $user->id, $icon, 'user', null, 'com_communitypolls', '', 'Polls');
                        }
                    }
                }
                // End Mighty Touch Support

                // Twitter Tweets Start
                if($jcp_config[ENABLE_TWITTER] == "1") {
                    PollsHelper::sendToTwitter($poll);
                }
                // Twitter Tweets End
                break;
        }
    }

    /**
     * Checks if the logged in user has permissions to publish/unpublish poll.
     * To get different user permission, override the user id by passing it as parameter.
     *
     * @param <type> $userid
     * @return <type>
     */
    function checkPublishPermissions($poll) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        return (CPAuthorization::authorize('com_communitypolls','manage','polls','all') ||
                        ((
                                !$user->guest &&
                                        $poll->created_by == $user->id) &&
                                $jcp_config[IS_USER_PUBLISH_ALLOWED])
        );
    }

    /**
     * Gets the IP address of the currently visiting user.
     *
     * @return <type>
     */
    function getUserIP() {
        $ip = false;
        if(!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        }
        if(!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ips = explode(", ", $_SERVER['HTTP_X_FORWARDED_FOR']);
            if($ip) {
                array_unshift($ips, $ip);
                $ip = false;
            }
            for($i = 0; $i < count($ips); $i++) {
                if(!preg_match("/^(10|172\.16|192\.168)\./i", $ips[$i])) {
                    if(version_compare(phpversion(), "5.0.0", ">=")) {
                        if(ip2long($ips[$i]) != false) {
                            $ip = $ips[$i];
                            break;
                        }
                    }
                    else {
                        if(ip2long($ips[$i]) != - 1) {
                            $ip = $ips[$i];
                            break;
                        }
                    }
                }
            }
        }
        return ($ip ? $ip : $_SERVER['REMOTE_ADDR']);
    }

    /**
     * Gets the avatar of the user. Currently supporting JomSocial, Kunena, AUP and CB.
     *
     * @global <type> $jcp_config
     * @param <type> $userid
     * @param <type> $height
     * @return <type>
     */
    function getUserAvtar($userid=0, $height=48) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db = JFactory::getDBO();
        $avatar = '';
        $username = $jcp_config[USER_NAME];
        switch ( $jcp_config["user_avtar"] ) {
            case 'jomsocial':
                $jspath = JPATH_BASE.DS.'components'.DS.'com_community';
                include_once($jspath.DS.'libraries'.DS.'core.php');

                // Get CUser object
                $jsuser =& CFactory::getUser($userid);
                $usrname = $jsuser->$username;
                $avatarLoc = $jsuser->getThumbAvatar();
                $link = CRoute::_('index.php? option=com_community&view=profile&userid='.$userid.'&Itemid='.JRequest::getInt('Itemid'));
                $avatar = '<a href="'.$link.'"><img class="hasTip" style="border: 1px solid #cccccc; height: '.$height.'px;" src="'.$avatarLoc.'" alt="'.$usrname.'" title="'.$usrname.'"/></a>';
                break;
            case 'cb':
                $strSql = "SELECT `avatar`, firstname FROM #__comprofiler WHERE `user_id`='{$userid}' AND `avatarapproved`='1'";
                $db->setQuery($strSql);
                $result = $db->loadObject();
                $link = JRoute::_( 'index.php?option=com_comprofiler&amp;task=userProfile&amp;user=' . $userid);
                if($result && !empty($result->avatar)) {
                    $avatarLoc = JURI::base(true)."/images/comprofiler/".$result->avatar;
                } else {
                    $avatarLoc = JURI::base(true)."/components/com_comprofiler/plugin/templates/default/images/avatar/nophoto_n.png";
                }
                $avatar = '<a href="'.$link.'"><img src="'.$avatarLoc.'" class="hasTip" style="border: 1px solid #cccccc; height: '.$height.'px;" alt="'.$result->firstname.'" title="'.$result->firstname.'"/></a>';
                break;
            case 'touch':
                $avatarLoc = JURI::base(true) . '/index2.php?option=com_community&amp;controller=profile&amp;task=avatar&amp;width=' . $height . '&amp;height=' . $height . '&amp;user_id=' . $userid . '&amp;no_ajax=1';
                $avatar = '<img src="' . $avatarLoc . '" style="border: 1px solid #cccccc; height: "' . $height . '"px;" alt=""/>';
                $link = JRoute::_("index.php?option=com_community&view=profile&user_id={$userid}&Itemid=".JRequest::getInt('Itemid'));
                $avatar = '<a href="'.$link.'">'.$avatar.'</a>';
                break;
            case 'gravatar':
                $strSql = 'SELECT email FROM #__users WHERE id=' . $userid;
                $db->setQuery($strSql);
                $email = $db->loadResult();
                $gravatar = 'http://www.gravatar.com/avatar.php?gravatar_id=';
                $gravatar .= md5( strtolower($email) );
                if ( $height ) {
                    $gravatar .= '&amp;size=$height';
                } else {
                    $gravatar .= '&amp;size=48';
                }
                $avatar = '<img src="'.$gravatar.'"/>';
                break;
            case 'kunena':
                $query = "SELECT a.avatar, b." . $username . " FROM #__fb_users as a" .
                        " LEFT JOIN #__users as b on b.id=a.userid where a.userid=".$userid;
                $db->setQuery( $query );
                $kunenaUser = $db->loadObject();
                $kunena_avtar = @$kunenaUser->avatar;
                $kunena_avtars = JURI::base(true) . '/images/fbfiles';
                if ($kunena_avtar != '') {
                    if(!file_exists( JPATH_ROOT .'/images/fbfiles/avatars/l_' . $kunena_avtar)) {
                        $avatarLoc = $kunena_avtars . '/avatars/' . $kunena_avtar;
                    }
                    else {
                        $avatarLoc = $kunena_avtars . '/avatars/l_' . $kunena_avtar;
                    }
                }
                else $avatarLoc = $kunena_avtars . '/avatars/nophoto.jpg';
                //URL
                $usrname = ($kunenaUser->$username)?$kunenaUser->$username:JText::_('GUEST');
                $link = JRoute::_( 'index.php?option=com_kunena&amp;func=fbprofile&amp;userid=' . $userid);
                $avatar = '<a href="'.$link.'" title="'.$usrname.'"><img src="'.$avatarLoc.'" class="hasTip" style="border: 1px solid #cccccc; height: '.$height.'px;" alt="'.$usrname.'"/></a>';
                break;
            case 'aup':
                $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                if ( file_exists($api_AUP)) {
                    require_once ($api_AUP);
                    $avatar = AlphaUserPointsHelper::getAupAvatar($userid, 1, $height, $height);
                }
                break;
        }
        return $avatar;
    }

    /**
     * Gets the avatar of the user. Currently supporting JomSocial, Kunena, AUP and CB.
     *
     * @global <type> $jcp_config
     * @param <type> $userid
     * @param <type> $height
     * @return <type>
     */
    function getPAUserAvtar($userid=0, $height=48) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db = JFactory::getDBO();
        $avatar = '';
        $username = $jcp_config[USER_NAME];
        $uri = substr(JURI::base(),0,-1);
        switch ( $jcp_config["user_avtar"] ) {
            case 'jomsocial':
                $jspath = JPATH_BASE.DS.'components'.DS.'com_community';
                include_once($jspath.DS.'libraries'.DS.'core.php');

                // Get CUser object
                $jsuser =& CFactory::getUser($userid);
                $usrname = $jsuser->$username;
                $avatarLoc = $jsuser->getThumbAvatar();

                $link = CRoute::_('index.php? option=com_community&view=profile&userid='.$userid.'&Itemid='.JRequest::getInt('Itemid'));
                $avatar = '<a href="'.$uri.$link.'"><img class="hasTip" style="border: 1px solid #cccccc; height: "' . 
                	$height . 'px;" src="'.$avatarLoc.'" alt="'.$usrname.'"  title="'.$usrname.'"/></a>';
                break;
            case 'cb':
                $strSql = "SELECT `avatar`, firstname FROM #__comprofiler WHERE `user_id`='{$userid}' AND `avatarapproved`='1'";
                $db->setQuery($strSql);
                $result = $db->loadObject();
                $link = JRoute::_( 'index.php?option=com_comprofiler&amp;task=userProfile&amp;user=' . $userid);
                if($result && !empty($result->avatar)) {
                    $avatarLoc = "/images/comprofiler/".$result->avatar;
                } else {
                    $avatarLoc = "/components/com_comprofiler/plugin/templates/default/images/avatar/nophoto_n.png";
                }
                $avatar = "<a href=\"" . $uri.$link . "\"><img src=\"" . $uri.$avatarLoc . "\"" .
                        "class=\"hasTip\" style=\"border: 1px solid #cccccc; height: " . $height . "px;\"" .
                        "alt=\"" . $result->firstname . "\" title=\"" . $result->firstname . "\"/></a>";
                break;
            case 'touch':
                $avatarLoc = '/index2.php?option=com_community&amp;controller=profile&amp;task=avatar&amp;width=' . $height . '&amp;height=' . $height . '&amp;user_id=' . $userid . '&amp;no_ajax=1';
                $avatar = '<img src="' . $uri.$avatarLoc . '" style="border: 1px solid #cccccc; height: "' . $height . '"px;" alt=""/>';
                $link = JRoute::_("index.php?option=com_community&view=profile&user_id={$userid}&Itemid=".JRequest::getInt('Itemid'));
                $avatar = '<a href="' . $uri.$link . '">' . $avatar . '</a>';
                break;
            case 'gravatar':
                $strSql = 'SELECT email FROM #__users WHERE id=' . $userid;
                $db->setQuery($strSql);
                $email = $db->loadResult();
                $gravatar = 'http://www.gravatar.com/avatar.php?gravatar_id=';
                $gravatar .= md5( strtolower($email) );
                if ( $height ) {
                    $gravatar .= '&amp;size=$height';
                } else {
                    $gravatar .= '&amp;size=48';
                }
                $avatar = '<img src="'.$gravatar.'"/>';
                break;
            case 'kunena':
                $query = "SELECT a.avatar, b." . $username . " FROM #__fb_users as a" .
                        " LEFT JOIN #__users as b on b.id=a.userid where a.userid=".$userid;
                $db->setQuery( $query );
                $kunenaUser = $db->loadObject();
                $kunena_avtar = @$kunenaUser->avatar;
                $kunena_avtars = '/images/fbfiles';
                if ($kunena_avtar != '') {
                    if(!file_exists( JPATH_ROOT .'/images/fbfiles/avatars/l_' . $kunena_avtar)) {
                        $avatarLoc = $kunena_avtars . '/avatars/' . $kunena_avtar;
                    }
                    else {
                        $avatarLoc = $kunena_avtars . '/avatars/l_' . $kunena_avtar;
                    }
                }
                else $avatarLoc = $kunena_avtars . '/avatars/nophoto.jpg';
                //URL
                $usrname = ($kunenaUser->$username)?$kunenaUser->$username:JText::_('GUEST');
                $link = JRoute::_( 'index.php?option=com_kunena&amp;func=fbprofile&amp;userid=' . $userid);
                $avatar = "<a href=\"" . $uri.$link . "\" title=\"" . $usrname . "\"><img src=\"" . $uri.$avatarLoc . "\"" .
                        "class=\"hasTip\" style=\"border: 1px solid #cccccc; height: " . $height . "px;\"" .
                        "alt=\"" . $usrname . "\"/></a>";
                break;
            case 'aup':
                $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                if ( file_exists($api_AUP)) {
                    require_once ($api_AUP);
                    $avatar = AlphaUserPointsHelper::getAupAvatar($userid, 1, $height, $height);
                }
                break;
        }
        return $avatar;
    }

    function sendToTwitter($poll) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $twitter_lib = JPATH_ROOT . DS . 'components' . DS . 'com_communitypolls' . DS . 'helpers' . DS . 'twitter.php';
        echo $twitter_lib;
        if(file_exists($twitter_lib)) {
            require_once($twitter_lib);
            try{
                $twi_user = new Twitter($jcp_config[TWITTER_USER_NAME], $jcp_config[TWITTER_PASSWORD]);
                $user =& JFactory::getUser();
                $poll_url =  JURI::root() . 'index.php?option=com_communitypolls&controller=polls&task=viewpoll&id='. $poll->id . ($poll->alias?(':' . $poll->alias):'');
                $poll_url = PollsHelper::getShortenedUrl($poll_url);
                $twi_text = $user->$jcp_config[USER_NAME] . ' ' . JText::_('TWEET_POLL') . ' ' . $poll_url . ' - ' . $poll->title;
                $twi_text = substr($twi_text, 0, 120);
                $twi_user->updateStatus($twi_text);
            }catch (TwitterException $e){}
        }
    }

    function getShortenedUrl($url) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $shortUrl;
        switch($jcp_config[SHORT_URL_SERVICE]) {
            case 'none':
                return $url;
            case 'bit.ly':
                require_once JPATH_ROOT . DS . 'components' . DS . 'com_communitypolls' . DS . 'helpers' . DS . 'bitly.php';
                $bitly = new Bitly($jcp_config[BIT_LY_LOGIN], $jcp_config[BIT_LY_API_KEY]);
                try {
                    $shortUrl = $bitly->shorten($url);
                }catch (BitlyException $e ) {
                    return false;
                }
                break;
            case 'tinyurl':
                require_once JPATH_ROOT . DS . 'components' . DS . 'com_communitypolls' . DS . 'helpers' . DS . 'tinyurl.php';
                $tinyUrl = new TinyUrl();
                $shortUrl = $tinyUrl->create($url);
                break;
            default:
                $shortUrl = $url;
                break;
        }
        return $shortUrl;
    }

    function getUserProfileUrl($userid=null, $username='Guest') {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db = JFactory::getDBO();
        $link = null;
        switch ( $jcp_config[USER_AVTAR] ) {
            case 'jomsocial':
                $jspath = JPATH_BASE.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php';
                if(file_exists($jspath)) {
                    include_once($jspath);
                    $link = '<a href="' . CRoute::_('index.php? option=com_community&view=profile&userid='.$userid) . '">' . $username . '</a>';
                }
                break;
            case 'cb':
                $link = '<a href="' . substr(JURI::base(),0,-1) . JRoute::_( 'index.php?option=com_comprofiler&amp;task=userProfile&amp;user=' . $userid) . '">' . $username . '</a>';
                break;
            case 'touch':
                $link = PollsHelper::getTouchPopup($userid, $username);
                break;
            case 'kunena':
                $link = '<a href="' . substr(JURI::base(),0,-1) . JRoute::_( 'index.php?option=com_kunena&amp;func=fbprofile&amp;userid=' . $userid) . '">' . $username . '</a>';
                break;
            case 'aup':
                $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                if ( file_exists($api_AUP)) {
                    require_once ($api_AUP);
                    $link = '<a href="' . AlphaUserPointsHelper::getAupLinkToProfil($userid) . '">' . $username . '</a>';
                }
                break;
            default:
                $link = null;
        }
        return (!$link)?(!empty($username)?$username:JText::_("LBL_GUEST")):$link;
    }

    function getTouchPopup($user_id, $user_name) {
        Static $capi = false;
        Static $api_enabled = true;

        if($api_enabled == false) return $user_name;

        if(!$capi) {
            $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
            if(file_exists($API)) {
                require_once $API;
                $capi = new JSCommunityApi();
            }
            else {
                $api_enabled = false;
                return $user_name;
            }
        }

        $params['width'] = 400;

        $links[0]->link = JRoute::_( 'index.php?option=com_communitypolls&controller=polls&task=userpolls&uid='.$user_id.'&Itemid='.JRequest::getInt('Itemid'));
        $links[0]->alt = JText::_('LBL_ALL_POLLS_FROM') . ' this user';
        $links[0]->icon = 'components/com_communitypolls/assets/images/poll_add.png';

        $user_name = $capi->getUserSlideMenu($user_id, $user_name, $links, $params);

        return $user_name;
    }
    
    function sendMail($from, $fromname, $recipient, $subject, $body, $mode=0, $cc=null, $bcc=null, $attachment=null, $replyto=null, $replytoname=null){
        // Get a JMail instance
        $mail = &JFactory::getMailer();

        $mail->setSender(array($from, $fromname));
        $mail->setSubject($subject);
        $mail->setBody($body);

        // Are we sending the email as HTML?
        if ($mode) {
            $mail->IsHTML(true);
        }

        $mail->addRecipient($recipient);
        $mail->addCC($cc);
        $mail->addBCC($bcc);
        $mail->addAttachment($attachment);

        // Take care of reply email addresses
        if (is_array($replyto)) {
            $numReplyTo = count($replyto);
            for ($i=0; $i < $numReplyTo; $i++){
                    $mail->addReplyTo(array($replyto[$i], $replytoname[$i]));
            }
        } elseif (isset($replyto)) {
            $mail->addReplyTo(array($replyto, $replytoname));
        }

        return  $mail->Send();
    }
        
    function getPoweredByLink() {
        $poweredby = '<div style="text-align: center; width: 100%; font-family: arial; font-size:9px; font-color: #ccc;">' . JText::_('POWERED_BY') . ' <a href="http://www.corejoomla.com" alt="CoreJoomla">Community Polls</a></div>';
        return $poweredby;
    }

	function getFormattedDate($strdate) {
        jimport('joomla.utilities.date');
        $user =& JFactory::getUser();
        if($user->get('id')) {
            $tz = $user->getParam('timezone');
        } else {
            $conf =& JFactory::getConfig();
            $tz = $conf->getValue('config.offset');
        }

        // Given time
        $jdate = new JDate($strdate);
        $jdate->setOffset($tz);
        $date = $jdate->toISO8601();
        
        return Date_Difference::getStringResolved($date);
	}
}