<?php
/**
 * Joomla! 1.5 component Community Polls
 *
 * @version $Id: communitypolls.php 2009-08-10 03:45:15 svn $
 * @author Maverick
 * @package Joomla
 * @subpackage Community Polls
 * @license GNU/GPL
 *
 * The Community Polls allows the members of the Joomla website to create and manage polls from the front-end.
 * The administrator has the powerful tools provided in the back-end to manage the polls published by all users.
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

/**
 * Community Polls Component Community Polls Model
 *
 * @author      maverick
 * @package		Joomla
 * @subpackage	Community Polls
 * @since 1.5
 */
class CommunityPollsModelPolls extends JModel {

    function __construct() {
        parent::__construct();
    }

    function &getPolls() {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();
        // Get pagination request variables
        $limit = $jcp_config[LIST_LENGTH];
        if(!$limit) {
            $limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
        }
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
        $catid = JRequest::getInt('catid',0);

        $filterby = $app->getUserStateFromRequest("$option.jcpfilter",'jcpfilter', 0);
        $sortby = $app->getUserStateFromRequest("$option.jcpsort",'jcpsort', 0);
        $searchby = $app->getUserStateFromRequest("$option.jcpsearch",'jcpsearch',null);

        $keywords = null;
        if($searchby) {
            $keys = explode(' ', $searchby);
            $count = 0;
            foreach ($keys as $key) {
                $key = trim($key);
                if(strlen($key) > 2) {
                    if($count == 0) {
                        $keywords = 'p.title LIKE "%' . $db->getEscaped($key) . '%"';
                    }else {
                        $keywords .= ' OR p.title LIKE "%' . $db->getEscaped($key) . '%"';
                    }
                    $count++;
                }
            }
        }

        $eligibility = '';
        if( (strpos($jcp_config[VOTE_RESTRICTION_METHOD],'username') !== FALSE) && !$user->guest) {
            $eligibility = ', (SELECT COUNT(*)<=0 FROM ' . TABLE_JCP_VOTES .
                    ' WHERE poll_id=p.id AND voter_id=' . $user->id . ' AND voter_id!=0) AS eligible';
        }else {
            $eligibility = ', 1 AS eligible';
        }

        $filter = $this->_filterBy($filterby);
        $sort   = $this->_sortBy($sortby);

        $where = ' WHERE p.published=1';
        if($catid) {
            $where .= ' AND category =' . $catid;
            $sort = ' ORDER BY featured DESC,' . $sort;
        }else {
            $sort = ' ORDER BY ' . $sort;
        }
        if($filter) {
            $where .= ' AND ' . $filter;
        }
        if($keywords) {
            $where .= ' AND (' . $keywords . ')';
        }
        $query = 'SELECT COUNT(*) FROM ' . TABLE_JCP_POLLS . ' p' . $where;
        $db->setQuery( $query );
        $total = $db->loadResult();
        jimport('joomla.html.pagination');
        $pagination = new JPagination( $total, $limitstart, $limit );

        $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                ' p.created as created, p.published, p.created_by, u.' . $jcp_config[USER_NAME] .
                ' AS username ' . $eligibility  . ', p.featured, p.close_date,' .
                ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                $where . $sort;
        $data = $this->_getList($query, $pagination->limitstart, $pagination->limit);
        $lists['jcpsearch'] = $searchby;
        $lists['jcpfilter'] = $filterby;
        $lists['jcpsort']   = $sortby;

        $result = new stdClass();
        $result->items      =& $data;
        $result->lists      =& $lists;
        $result->pagination =& $pagination;

        return $result;
    }

    function &getJomCommentCount($cids, $option) {
        if($cids) {
            $db =& $this->getDBO();
            $cid = implode(",", $cids);
            $query = "SELECT COUNT(*) comments, contentid FROM #__jomcomment WHERE `contentid` IN ($cid)
                        AND (`option`='$option') AND `published`='1' GROUP BY contentid order by contentid";
            $db->setQuery($query);
            $list = $db->loadObjectList();
            return $list;
        }else {
            return null;
        }
    }

    function &getCategories() {
        $db =& $this->getDBO();
        $category = JRequest::getInt('catid', 0);
        $where = ($category) ? ' AND id=' . $category : '';
        $query = 'SELECT id, title, alias, published, locked, num_polls, num_votes FROM ' .
                TABLE_JCP_CATEGORIES . ' WHERE published=1' . $where . ' ORDER BY ordering';
        $db->setQuery($query);
        $list = $db->loadObjectList();
        return $list;
    }

    function castVote($poll_id=0,$options=array()) {
        $db =& JFactory::getDBO();
        $user =& JFactory::getUser();
        if(!$poll_id) {
            $poll_id = JRequest::getInt('id',0);
        }
        if(count($options)<=0) {
            $this->setError(JText::_('MSG_ERROR_NO_SELECTION'));
            return false;
        }else{
        	JArrayHelper::toInteger($options,0);
        }
        $poll = new JObject();
        $poll->id = $poll_id;
        $poll->close_date = "";
        $eligible = $this->traceEligibility($poll, false);

        if($eligible) {
            $ip = PollsHelper::getUserIP();
            $userid = ($user && $user->id)?$user->id:0;
            $query = 'INSERT INTO ' . TABLE_JCP_VOTES . '(poll_id, voter_id, voted_on,ip_address,option_id) VALUES ';
            foreach($options as $vote) {
            	$query = $query . '(' . $poll_id . ',' . $userid . ', NOW(),' . $db->quote($ip) . ',' . $vote . '),';
            }
            $query = substr($query, 0, -1);
            $db->setQuery( $query );
            if(!$db->query()) {
                $this->setError(JText::_('MSG_ERROR_ON_VOTE') . ". Error Code: 100010".$query);
                return false;
            }
            
            $query = 'UPDATE ' . TABLE_JCP_POLLS . ' SET voters=voters+1, votes=votes+' . count($options) . ', last_voted=NOW() WHERE id=' . $poll_id;
            $db->setQuery( $query );
            if(!$db->query()) {
                $this->setError(JText::_('MSG_ERROR_ON_VOTE'). ". Error Code: 100011");
                return false;
            }
            
            $options_csv = implode(",", $options);
            $query = 'UPDATE ' . TABLE_JCP_OPTIONS . ' SET votes=votes+1 WHERE id IN(' . $options_csv . ')';
            $db->setQuery( $query );
			if(!$db->query()) {
                $this->setError(JText::_('MSG_ERROR_ON_VOTE') . ". Error Code: 100012");
            	return false;
            }
            
            $query = 'UPDATE ' . TABLE_JCP_CATEGORIES . ' c SET c.num_votes=c.num_votes+'.count($options).' WHERE c.id IN(' .
                    ' SELECT category FROM ' . TABLE_JCP_POLLS . ' p WHERE p.id=' . $poll_id . ')';
            $db->setQuery($query);
            $db->query();

            return true;
        }else {
            $this->setError(JText::_('MSG_ALREADY_VOTED'));
            return false;
        }
    }

    function castGridVote($poll_id,$votes) {
        $db =& JFactory::getDBO();
        $user =& JFactory::getUser();
        if(!$poll_id || !$votes || count($votes)<=0) {
            $this->setError(JText::_('MSG_ERROR_NO_SELECTION'));
            return false;
        }
        
        $poll = new JObject();
        $poll->id = $poll_id;
        $eligible = $this->traceEligibility($poll, false);

        if($eligible) {
            $query = 'UPDATE ' . TABLE_JCP_POLLS . ' SET voters=voters+1, votes=votes+'.count($votes).', last_voted=NOW() WHERE id=' . $poll_id;
            $db->setQuery( $query );
            if(!$db->query()) {
                $this->setError(JText::_('MSG_ERROR_ON_VOTE'));
                return false;
            }

            $ip = PollsHelper::getUserIP();
           
            $query = 'INSERT INTO ' . TABLE_JCP_VOTES . '(poll_id, voter_id, voted_on, ip_address, option_id, column_id) VALUES ';
            $sVotes='';
            foreach ($votes as $vote){
                $voteEx = explode('_', $vote);
                $row = (int)$voteEx[0];
                $column = (int)$voteEx[1];
                $sVotes = $sVotes . $row . ',';
                $query .= '(' . $poll_id . ',' . $user->id . ', NOW(), ' . $db->quote($ip) . ',' . $row . ',' . $column . '),';
            }

            $query = substr($query, 0, -1);
			$sVotes = substr($sVotes, 0, -1);
			
            $db->setQuery( $query );
            if(!$db->query()) {
                $query2 = 'UPDATE ' . TABLE_JCP_OPTIONS . ' SET votes=votes+1 WHERE id IN(' . $sVotes . ') AND poll_id=' . $poll_id;
                $db->setQuery( $query2 );
                if(!$db->query()) {
                    $this->setError(JText::_('MSG_ERROR_ON_VOTE'));
                    return false;
                }
                $this->setError(JText::_('MSG_ERROR_ON_VOTE'));
                return false;
            }
            $query = 'UPDATE ' . TABLE_JCP_CATEGORIES . ' c SET c.num_votes=c.num_votes+'.count(votes).' WHERE c.id IN(' .
                    ' SELECT category FROM ' . TABLE_JCP_POLLS . ' p WHERE p.id=' . $poll_id . ')';
            $db->setQuery($query);
            $db->query();

            return true;
        }else {
            $this->setError(JText::_('MSG_ALREADY_VOTED'));
            return false;
        }
    }

    function getUsersVoted($pollid) {
        $app = &JFactory::getApplication();
        $jcp_config = &$app->getUserState( SESSION_JCP_CONFIG );
        $db =& $this->getDBO();
        $username = (!empty($jcp_config[USER_NAME]))?$jcp_config[USER_NAME]:'username';
        $query = 'SELECT distinct(u.' . $username . ') as username, u.id, v.voted_on, v.ip_address FROM ' .
                TABLE_JCP_VOTES . ' v LEFT JOIN #__users u ON v.voter_id=u.id and u.id > 0'.
                ' WHERE v.voter_id != 0 AND v.poll_id=' . $pollid . ' ORDER BY v.voted_on DESC';
        $db->setQuery($query);
        $users = $db->loadObjectList();
        return $users;
    }

    function &addNewPoll() {
    	global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db =& JFactory::getDBO();
        $user =& JFactory::getUser();

        $title = trim(JRequest::getVar('polltitle','','post','STRING'));
        $options = JRequest::getVar('pollOptions','','post','ARRAY');
        $columns = JRequest::getVar('pollColumn','','post','ARRAY');
        $category = JRequest::getVar('catid',0,'post','INT');
        $closingdate = JRequest::getVar('closingdate','','post','STRING');
        $images = JRequest::getVar('imageAttachment', null,'post','ARRAY');
        $poll_type = JRequest::getVar('poll_type','radio','post','WORD');
        $polls_anywhere = JRequest::getVar('polls_anywhere',0,'post','INT');
        if (CPAuthorization::authorize($option, 'wysiwyg', 'polls', 'all')) {
            $description = JRequest::getVar('description', '', 'post', 'string', JREQUEST_ALLOWRAW);
        } else {
            $description = JRequest::getVar('description', '', 'post', 'string');
        }
        $description = $this->filterData($description);
        
        if($polls_anywhere){
        	$polls_anywhere = 1;
        }else{
        	$polls_anywhere = 0;
        }
        if(!isset($options) || (count($options) < 0) || (strlen($title) < 1)) {
            $this->setError(JText::_('MSG_ERROR_SAVE_POLL'));
            return false;
        }

        if(!in_array($poll_type, array('radio','checkbox','grid'))) {
            $poll_type = 'radio';
        }
        $todays_date = date("Y-m-d");
        $today = strtotime($todays_date);
        $close_date = strtotime($closingdate);

        if($close_date <= $today) {
            $closingdate = 'null';
        }else {
            $closingdate = $db->Quote($closingdate);
        }

        jimport( 'joomla.filter.output' );
        $alias = JFilterOutput::stringURLSafe($title);

        $poll_id = 0;
        if($jcp_config[IS_MODERATED] == '1' || ($jcp_config[IS_MODERATED] == '2' && $user->guest)) {
            $status = "2";
        }else {
            $status = "1";
        }
        $ip = PollsHelper::getUserIP();
        $query = 'INSERT INTO ' . TABLE_JCP_POLLS . '(title, alias, description, created,' .
                	' created_by, category, published,close_date,ip_address, type, anywhere)' .
                ' SELECT ' . 
        			$db->Quote($title) . ',' . 
        			$db->Quote($alias) . ',' . 
        			$db->Quote($description) . ',' .
                	$db->quote ( gmdate ( 'Y-m-d H:i' ) ) . ',' . 
        			$user->id . ',' . 
        			$category. ',' . 
        			$status . ',' . 
        			$closingdate . ',' . 
        			$db->Quote($ip) . ',' . 
        			$db->Quote($poll_type) . ',' . 
        			$polls_anywhere . 
        		' FROM ' .
                	TABLE_JCP_CATEGORIES . ' WHERE id=' . $category . ' AND published=1 AND locked=0';
        $db->setQuery($query);

        $poll = new stdClass();
        if(!$db->query()) {
            $this->setError(JText::_('MSG_ERROR_SAVE_POLL'));
            return false;
        }else {
            $poll_id = $db->insertid();
            if(!$poll_id) {
                $this->setError(JText::_('MSG_ERROR_SAVE_POLL'));
                return false;
            }

            foreach($options as $polloption) {
                $option_elements = explode('|', trim($polloption));
                if((count($option_elements) < 2) || (strlen($option_elements[1]) < 1)) {
                    continue;
                }
                $query = 'INSERT INTO ' . TABLE_JCP_OPTIONS . '(poll_id, title, type) VALUES ' .
                        '(' . $poll_id . ',' . $db->quote($option_elements[1]) . ','. $db->quote('x') .')';
                $db->setQuery($query);
                if(!$db->query()) {
                    $this->setError(JText::_('MSG_ERROR_SAVE_POLL'));
                    return false;
                }
                $db_optionid = $db->insertid();

                if(isset($images) && (count($images) > 0)) {
                    foreach ($images as $image) {
                        $image_elements = explode('|',trim($image));
                        if(count($image_elements) < 2) {
                            continue;
                        }else {
                            if(strcmp($option_elements[0], $image_elements[0]) == 0) {
                                $query = 'INSERT INTO ' . TABLE_JCP_RESOURCES . '(type, value, poll_id, option_id) VALUES(' .
                                        $db->quote('image') . ',' .
                                        $db->quote($image_elements[1]) . ',' .
                                        $poll_id . ',' .
                                        $db_optionid . ')';
                                $db->setQuery($query);
                                if(!$db->query()) {
                                    $this->setError(JText::_('MSG_ERROR_SAVE_POLL'));
                                    return false;
                                }
                                break;
                            }
                        }
                    }
                }
            }
            if(strcmp($poll_type,'grid') == 0) {
                if(isset($columns) && (count($columns) > 0)) {
                    foreach($columns as $column) {
                        if(strlen(trim($column)) > 0) {
                            $query = 'INSERT INTO ' . TABLE_JCP_OPTIONS . '(poll_id, title, type) VALUES ' .
                                    '(' . $poll_id . ',' . $db->quote($column) . ','. $db->quote('y') .')';
                            $db->setQuery($query);
                            if(!$db->query()) {
                                $this->setError(JText::_('MSG_ERROR_SAVE_POLL'));
                                return false;
                            }
                        }
                    }
                }
            }

            $query = 'UPDATE ' . TABLE_JCP_CATEGORIES . ' SET num_polls=num_polls+1 WHERE id=' . $category;
            $db->setQuery($query);
            $db->query();

            $poll->id = $poll_id;
            $poll->title = $title;
            $poll->alias = $alias;
        }
        return $poll;
    }

    function filterData($data) {
        // Filter settings
        jimport('joomla.application.component.helper');
        $config = JComponentHelper::getParams('com_content');
        $user = &JFactory::getUser();
        $gid = $user->get('gid');

        $filterGroups = $config->get('filter_groups');

        // convert to array if one group selected
        if ((!is_array($filterGroups) && (int) $filterGroups > 0)) {
            $filterGroups = array($filterGroups);
        }

        if (is_array($filterGroups) && in_array($gid, $filterGroups)) {
            $filterType = $config->get('filter_type');
            $filterTags = preg_split('#[,\s]+#', trim($config->get('filter_tags')));
            $filterAttrs = preg_split('#[,\s]+#', trim($config->get('filter_attritbutes')));
            switch ($filterType) {
                case 'NH':
                    $filter = new JFilterInput();
                    break;
                case 'WL':
                    $filter = new JFilterInput($filterTags, $filterAttrs, 0, 0);
                    break;
                case 'BL':
                default:
                    $filter = new JFilterInput($filterTags, $filterAttrs, 1, 1);
                    break;
            }
            $data = $filter->clean($data);
        } elseif (empty($filterGroups)) {
            $filter = new JFilterInput(array(), array(), 1, 1);
            $data = $filter->clean($data);
        }
        return $data;
    }
    
    function uploadNewFile() {
        $filetype = $_FILES['uploadFile']['type'];
        $gd_function_suffix = array(
                'image/pjpeg' => 'JPEG',
                'image/jpeg' => 'JPEG',
                'image/gif' => 'GIF',
                'image/bmp' => 'WBMP',
                'image/png' => 'PNG',
                'image/x-png' => 'PNG'
        );
        $function_suffix = $gd_function_suffix[$filetype];

        // Build Function name for ImageCreateFromSUFFIX
        if(!$function_suffix) {
            $err = "Unsupported file type, must be JPG|PNG|GIF|BMP";
        }else {
            $function_to_read = 'ImageCreateFrom' . $function_suffix;

            // Get the image and create a thumbnail
            $img = $function_to_read($_FILES['uploadFile']['tmp_name']);
            if (!$img) {
                $err =  "could not create image handle";
            }

            $width = imageSX($img);  // Get the image width
            $height = imageSY($img); // Get the image height

            if (!$width || !$height) {
                $err =  "No Width or Height";
            }

            // Build the thumbnail
            $target_width = 100;  // Thumnail width
            $target_height = 100; // Thumbnail height
            $target_ratio = $target_width / $target_height;  //New target ratio

            $img_ratio = $width / $height;  // Original ratio

            if ($target_ratio > $img_ratio) {
                $new_height = $target_height;
                $new_width = $img_ratio * $target_height; // Tall Image
            } else {
                $new_height = $target_width / $img_ratio; // Wide Image
                $new_width = $target_width;
            }

            if ($new_height > $target_height) {
                $new_height = $target_height;
            }
            if ($new_width > $target_width) {
                $new_height = $target_width;
            }

            $new_img = ImageCreateTrueColor($target_width, $target_height); // Create a new image handle for thumbnail
            $white = imagecolorallocate($new_img, 255, 255, 255); // Allocate the color white
            if (!@imagefilledrectangle($new_img, 0, 0, $target_width-1, $target_height-1, $white)) {	// Fill the image white
                $err = "Couldn't fill image";
            }
            // Copy the uploaded image to the newly created image
            if (!@imagecopyresampled($new_img, $img, ($target_width-$new_width)/2, ($target_height-$new_height)/2, 0, 0, $new_width, $new_height, $width, $height)) {
                $err = "couldn't copy new image";
            }

            // Use a output buffering to load the image into a variable
            ob_start();
            imagejpeg($new_img);
            ob_end_clean();

            // Create a unique id or name for the image
            $file_id = md5($_FILES["uploadFile"]["tmp_name"] + rand()*100000);
            $new_name = $file_id.".$function_suffix";
            $temp_name = $_FILES["uploadFile"]["tmp_name"];

            // assign upload directory, and target path for image
            $uploaddir = getcwd(); //
            $target_path = $uploaddir.DS.$new_name;

            // Move uploaded image to uploaddir location
            if(!@move_uploaded_file($temp_name, $target_path)) {
                $err = "Couldn't Copy File to Filesystem";
            }
        }
        // If no error, then pass good result and file id to Javascript for generating the thumbnail.
        // Pass our $err varialbe along

        if(!$err) {
            $result = "'1','".$file_id."'";
        }else {
            $result = "'.$err.','".$file_id."'";
        }
    }

    function getAjxPollData($id){
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db =& JFactory::getDBO();
        if(!$id){ return false; }
        $query = 'SELECT p.id, p.title, p.alias, p.description, p.votes, p.type,' .
                ' p.published, p.created, p.created_by, p.featured, p.close_date,'.
                ' u.' . $jcp_config[USER_NAME] . ' AS username, p.category as catid,' .
                ' jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                ' LEFT JOIN #__users u ON p.created_by=u.id WHERE p.published=1 and p.id=' . $id;
        $db->setQuery($query);
        $poll = $db->loadObject();
        if(isset($poll)) {
            if(!in_array($poll->type, array('radio','checkbox'))){
                return false;
            }
            $query = "SELECT id, title, votes, type FROM " . TABLE_JCP_OPTIONS .
                    " WHERE poll_id=" . $id . " AND title IS NOT NULL AND title <> ''";
            $db->setQuery($query);
            $answers = $db->loadObjectList();
            if(!isset($answers) || count($answers) < 1) {
                return false;
            }
            $poll->answers = $answers;
            return $poll;
        }else{
            return false;
        }
    }

    function doAjxVoteAnywhere($id,$votes){
        $db =& JFactory::getDBO();
        $poll = new JObject();
        $poll->id = $id;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        
        $query = "select p.anywhere from " . TABLE_JCP_POLLS . " p where p.id=" . $id;
        $db->setQuery($query);
        $anywhere = $db->loadResult();
        if($anywhere == '2' || (($anywhere == '0') && ($jcp_config[ENABLE_PA_BY_DEFAULT] == '0'))){
        	$this->setError(JText::_('MSG_ERROR_ON_VOTE') . ": Polls Anywhere is not active on this poll : 10110");
        	return false;
        }
        
        $query = 'UPDATE ' . TABLE_JCP_POLLS . ' SET voters=voters+1, votes=votes+' . count($votes) . ', last_voted=NOW() WHERE id=' . $id;
        $db->setQuery( $query );
        if(!$db->query()) {
            $this->setError(JText::_('MSG_ERROR_ON_VOTE') . ": 10100");
            return false;
        }
        $votesX = implode(",", $votes);
        $ip = PollsHelper::getUserIP();

        $query2 = 'UPDATE ' . TABLE_JCP_OPTIONS . ' SET votes=votes+1 WHERE id IN(' . $votesX . ') AND poll_id=' . $id;
        $db->setQuery( $query2 );
        if(!$db->query()) {
            $this->setError(JText::_('MSG_ERROR_ON_VOTE') . ": 10101" . $db->getErrorMsg());
            return false;
        }
        $query = 'INSERT INTO ' . TABLE_JCP_VOTES . '(poll_id, voter_id, voted_on,ip_address,option_id) VALUES ';
        foreach($votes as $vote) {
            $query .= '(' . $id . ',0, NOW(),"' . $ip . '",' . $vote . '),';
        }
        $query = substr($query, 0, (strlen($query) - 1));
        $db->setQuery( $query );
        if(!$db->query()) {
            $this->setError(JText::_('MSG_ERROR_ON_VOTE') . ": 10102" . $db->getErrorMsg());
            return false;
        }
        
        $query = 'UPDATE ' . TABLE_JCP_CATEGORIES . ' c SET c.num_votes=c.num_votes+'.count($votes).' WHERE c.id IN(' .
                ' SELECT category FROM ' . TABLE_JCP_POLLS . ' p WHERE p.id=' . $id . ')';
        $db->setQuery($query);
        $db->query();

        $query = "SELECT id, title, votes FROM " . TABLE_JCP_OPTIONS .
                    " WHERE poll_id=" . $id . " AND title IS NOT NULL AND title <> ''";
        $db->setQuery($query);
        $answers = $db->loadObjectList();
        if(!isset($answers) || count($answers) < 1) {
            return false;
        }
        $poll->answers = $answers;
        return $poll;
    }

    function getPollData($poll_id = 0, $poll_only=false) {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db =& JFactory::getDBO();
        $user =& JFactory::getUser();
        if(!$poll_id) {
            $poll_id = JRequest::getVar('id',0,'','INTEGER');
        }

        $query = 'SELECT p.id, p.title, p.alias, p.description, p.votes, p.type,' .
                ' p.published, p.created, p.created_by, p.featured, p.close_date,'.
                ' u.' . $jcp_config[USER_NAME] . ' AS username, p.category as catid,' .
                ' jc.title AS category, p.close_date, p.anywhere FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                ' LEFT JOIN #__users u ON p.created_by=u.id WHERE p.id=' . $poll_id;
        $db->setQuery($query);
        $poll = $db->loadObject();

        if(isset($poll)) {
            // Get the options
            $query = "SELECT id, title, votes, type FROM " . TABLE_JCP_OPTIONS . " WHERE poll_id=" . $poll_id . " AND title IS NOT NULL AND title <> '' ORDER BY id ASC";
            $db->setQuery($query);
            $options = $db->loadObjectList();

            if(!isset($options) || count($options) < 1) {
                $this->setError(JText::_('MSG_NO_OPTIONS'));
                return false;
            }
            $query = "SELECT type, option_id, value FROM " . TABLE_JCP_RESOURCES . " WHERE poll_id=" . $poll_id . " AND value IS NOT NULL AND value <> ''";
            $db->setQuery($query);
            $resources = $db->loadObjectList();

            if($resources && count($resources)){
                for($i=0; $i<count($options);$i++) {
                    foreach($resources as $resource) {
                        if($resource->option_id == $options[$i]->id) {
                            $options[$i]->resources[] = $resource;
                            break;
                        }
                    }
                }
            }
            
            $poll->options =& $options;
            $poll->eligible = $this->traceEligibility($poll, false);

            if(!$poll_only){
	            $where = '';
	            $catid = JRequest::getInt('catid',0);
	            if($catid) {
	                $where = ' AND category=' . $catid;
	            }
            	// Get the next item
	            $query = 'SELECT p.id, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
	                    ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug FROM ' .
	                    TABLE_JCP_POLLS . ' p WHERE p.published=1 AND p.id < ' . $poll_id .
	                    $where . ' ORDER BY p.id DESC LIMIT 1';
	
	            $db->setQuery($query);
	            $poll->next = $db->loadObject();
	
	            // Get the previous item
	            $query = 'SELECT id, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
	                    ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug FROM ' .
	                    TABLE_JCP_POLLS . ' p WHERE p.published=1 AND p.id > ' . $poll_id .
	                    $where . ' ORDER BY p.id ASC LIMIT 1';
	
	            $db->setQuery($query);
	            $poll->previous = $db->loadObject();
	            
		        if($jcp_config[AUTHOR_POLLS_COUNT] > 0) {
		            $poll->authorpolls =& $this->getAuthorPolls($poll->created_by, $poll_id);
		        }
		        if($jcp_config[FEATURED_POLLS_COUNT] > 0) {
		            $poll->featuredpolls =& $this->getFeaturedPolls();
		        }
		        if($jcp_config[RELATED_POLLS_COUNT] > 0) {
		            $poll->relatedpolls =& $this->getRelatedPolls($poll->title, $poll->id);
		        }
		        if($jcp_config[LATEST_POLLS_COUNT] > 0) {
		            $poll->latestpolls =& $this->getLatestPolls();
		        }
		        if($jcp_config[MOSTVOTED_POLLS_COUNT] > 0) {
		            $poll->mostvotedpolls =& $this->getMostVotedPolls();
		        }
	        }
            
            if(strcmp($poll->type, 'grid') == 0){
                $query = 'SELECT count(*) as votes, option_id , column_id FROM ' . TABLE_JCP_VOTES .
                    ' WHERE poll_id=' . $poll->id . ' GROUP BY option_id, column_id';
                $db->setQuery($query);
                $votes = $db->loadObjectList();
                $poll->votesEx = $votes;
            }
        }else {
            $this->setError(JText::_('MSG_POLL_NOT_EXIST'));
            return false;
        }
        return $poll;
    }

    function &getRelatedPolls($title, $pollid) {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();
        $excludes = $jcp_config[RELATED_EXCLUDED_WORDS];

        $keywords = null;
        $keys = explode(' ', $title);
        $count = 0;
        foreach ($keys as $key) {
            $key = trim($key);
            if(!stristr($excludes, ','.$key.',')) {
                if(strlen($key) >= $jcp_config[RELATED_KEY_LEN]) {
                    if($count == 0) {
                        $keywords = 'p.title LIKE "%' . $db->getEscaped($key) . '%"';
                    }else {
                        $keywords .= ' OR p.title LIKE "%' . $db->getEscaped($key) . '%"';
                    }
                    $count++;
                }
            }
        }

        $eligibility = '';
        if($user && $user->id > 0) {
            $eligibility = ', (SELECT COUNT(*) FROM ' . TABLE_JCP_VOTES .
                    ' WHERE poll_id=p.id AND voter_id=' . $user->id . ') AS eligible';
        }else {
            $eligibility = ', 1 AS eligible';
        }

        $where = ' WHERE p.published=1 AND p.id != ' . $pollid;
        if($keywords) {
            $where .= ' AND (' . $keywords . ')';
        }
        $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                ' p.created as created, p.published, p.created_by, u.username' . $eligibility  . ',' .
                ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' . $where;
        $data = $this->_getList($query, 0, $jcp_config[RELATED_POLLS_COUNT]);

        return $data;
    }

    function &getLatestPolls() {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();

        $where = ' WHERE p.published=1';
        $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                ' p.created as created, p.published, p.created_by, u.username,' .
                ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                $where . ' ORDER BY p.created DESC';
        $data = $this->_getList($query, 0, $jcp_config[LATEST_POLLS_COUNT]);

        return $data;
    }

    function &getMostVotedPolls() {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();

        $where = ' WHERE p.published=1';
        $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                ' p.created as created, p.published, p.created_by, u.username,' .
                ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                $where . ' ORDER BY p.votes DESC';
        $data = $this->_getList($query, 0, $jcp_config[MOSTVOTED_POLLS_COUNT]);

        return $data;
    }

    function &getFeaturedPolls() {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();

        $where = ' WHERE p.published=1 AND p.featured=1';
        $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                ' p.created as created, p.published, p.created_by, u.username,' .
                ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                $where . ' ORDER BY p.created DESC';
        $data = $this->_getList($query, 0, $jcp_config[FEATURED_POLLS_COUNT]);

        return $data;
    }

    function &getAuthorPolls($userid, $cid) {
        global $option;
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();

        $where = ' WHERE p.published=1 AND p.created_by=' . $userid . ' AND p.id <> ' . $cid;
        $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                ' p.created as created, p.published, p.created_by, u.username,' .
                ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                $where . ' ORDER BY p.created DESC';
        $data = $this->_getList($query, 0, $jcp_config[AUTHOR_POLLS_COUNT]);

        return $data;
    }

    function traceEligibility($poll, $skipDB) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $user =& JFactory::getUser();
        $db =& JFactory::getDBO();
        if(!$poll->id) {
            return false;
        }

        // Check if the poll is closed for voting.
        if($poll->close_date) {
            $todays_date = date("Y-m-d");
            $today = strtotime($todays_date);
            $close_date = strtotime($poll->close_date);
            if($close_date < $today) {
                return false;
            }
        }
        
        if(strpos($jcp_config[VOTE_RESTRICTION_METHOD],'cookie') !== FALSE){
        	$cookieName	= JUtility::getHash( $app->getName() . COOKIE_PREFIX . $poll->id );
        	if(isset($_COOKIE[$cookieName])) {
        		return false;
        	}
        }
        
        if(strpos($jcp_config[VOTE_RESTRICTION_METHOD],'username') !== FALSE && !$user->guest && !$skipDB){
			$query = 'SELECT COUNT(*) FROM ' . TABLE_JCP_VOTES . ' WHERE poll_id=' .$poll->id . ' AND voter_id=' . $user->id . ' AND voter_id > 0';
			$db->setQuery($query);
			$count = $db->loadResult();
			if($count > 0) {
				return false;
			}
        }
        
        if((strpos($jcp_config[VOTE_RESTRICTION_METHOD],'ip') !== FALSE) && !$skipDB){
        	$query = "SELECT COUNT(*) FROM " . TABLE_JCP_VOTES . " WHERE poll_id=" . $poll->id . " AND ip_address=" . $db->Quote(PollsHelper::getUserIP());
        	$db->setQuery($query);
        	$db->setQuery($query);
        	$count = $db->loadResult();
        	if($count > 0) {
        		return false;
        	}
        }

        return true;
    }

    function publish($state, $cid) {
        $app = &JFactory::getApplication();
        $jcp_config = $app->getUserState( SESSION_JCP_CONFIG );
        $db         =& $this->getDBO();
        $user =& JFactory::getUser();
        $query = 'UPDATE ' . TABLE_JCP_POLLS . ' SET published=' . $db->getEscaped($state) .
                ' WHERE id=' . $db->getEscaped($cid);
        $db->setQuery($query);
        if($db->query()) {
            return true;
        }else {
            return false;
        }
    }

    function locked($state, $cid) {
        $db         =& $this->getDBO();
        $query = 'UPDATE ' . TABLE_JCP_POLLS . ' SET locked=' . $db->getEscaped($state) .
                ' WHERE id =' . $db->getEscaped($cid) ;
        $db->setQuery($query);
        if($db->query()) {
            return true;
        }else {
            return false;
        }
    }

    function setFeaturedBit($state, $cid) {
        $db =& $this->getDBO();
        $query = 'UPDATE ' . TABLE_JCP_POLLS . ' SET featured=' . $state .
                ' WHERE id =' . (int)$cid ;
        $db->setQuery($query);
        if($db->query()) {
            return true;
        }else {
            return false;
        }
    }

    function isAllowedToCreatePoll($maxpolls) {
        $db     =& $this->getDBO();
        $user   =& JFactory::getUser();

        $query = 'SELECT COUNT(*) FROM ' . TABLE_JCP_POLLS . ' WHERE created_by=' . $user->id . ' AND created >= CURDATE() AND created_by > 0';
        $db->setQuery($query);
        $countpolls = (int)$db->loadResult();
        if($maxpolls && $countpolls >= $maxpolls) {
            return false;
        }else {
            return true;
        }
    }

    function _sortBy($sort) {
        $sortBy = '';
        switch($sort) {
            case 0:
                $sortBy = 'p.created DESC';
                break;
            case 1:
                $sortBy = 'p.created ASC';
                break;
            case 2:
                $sortBy = 'p.votes DESC';
                break;
            case 3:
                $sortBy = 'p.votes ASC';
                break;
        }

        return $sortBy;
    }

    function _filterBy($filter) {
        $where = null;
        switch($filter) {
            case 0:
            // All time
                $where = null;
                break;
            case 1:
            // Today
                $where = 'p.created >= CURDATE()';
                break;
            case 2:
            // Yesterday
                $where = 'p.created >= CURDATE()-1 AND p.created < CURDATE()';
                break;
            case 3:
            // This week
                $where = 'p.created >= SUBDATE(CURDATE(), INTERVAL WEEKDAY(NOW()) DAY)';
                break;
            case 4:
            // Last Week
                $where = 'p.created >= SUBDATE(CURDATE(), INTERVAL 7+WEEKDAY(NOW()) DAY)' .
                        ' AND p.created < SUBDATE(CURDATE(), INTERVAL WEEKDAY(NOW()) DAY)';
                break;
            case 5:
            // Last 7 days
                $where = 'p.created >= DATE_SUB(CURRENT_DATE, INTERVAL 6 DAY)';
                break;
            case 6:
            // This month
                $where = "p.created >= DATE_FORMAT(NOW(), '%Y-%m-01')";
                break;
            case 7:
            // Last month
                $where = "p.created >= DATE_SUB(DATE_FORMAT(NOW(), '%Y-%m-01'), INTERVAL 1 MONTH)" .
                        "AND  p.created < DATE_FORMAT(NOW(), '%Y-%m-01')";
                break;
            case 8:
            // Last 30 days
                $where = 'p.created > DATE_SUB(CURRENT_DATE, INTERVAL 30 DAY)';
                break;
            case 9:
            // Last 3 months
                $where = "p.created >= DATE_SUB(CURRENT_DATE, INTERVAL 3 MONTH)";
                break;
            case 10:
            // This year
                $where = "p.created >= DATE_FORMAT(NOW(), '%Y-01-%d')";
                break;
        }

        return $where;
    }

    function _searchBy($text) {

        $keywordArray = explode(" ", $text);
        $keywords = Array();
        foreach($keywordArray as $keyword) {
            if(strlen($keyword) > 2) {
                $keywords[] = $keyword;
            }
        }
        $keywordString = implode("p.title LIKE", $keywords);
        return $keywordString;
    }
}
?>