<?php
/**
 * Joomla! 1.5 component Community Polls
 *
 * @version $Id: communitypolls.php 2009-08-10 03:45:15 svn $
 * @author Maverick
 * @package Joomla
 * @subpackage Community Polls
 * @license GNU/GPL
 *
 * The Community Polls allows the members of the Joomla website to create and manage polls from the front-end.
 * The administrator has the powerful tools provided in the back-end to manage the polls published by all users.
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.model');

/**
 * Community Polls Component Community Polls Model
 *
 * @author      maverick
 * @package		Joomla
 * @subpackage	Community Polls
 * @since 1.5
 */
class CommunityPollsModelUser extends JModel {

    var $_error = '';
    var $_pagination = null;
    var $_total = null;
    var $_data = null;
    /**
     * Constructor
     */
    function __construct() {
        parent::__construct();
        $app = &JFactory::getApplication();

        // Get pagination request variables
        $limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'), 'int');
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);
    }

    function &getPollsCreatedByUser($userids=null) {
        global $option;
        $app = &JFactory::getApplication();
        $user =& JFactory::getUser();

        $filter_order       = $app->getUserStateFromRequest( "$option.filter_order",	'filter_order',		'p.created_by',	'cmd' );
        $filter_order_Dir   = $app->getUserStateFromRequest( "$option.filter_order_Dir",	'filter_order_Dir',	'DESC', 'word' );
        $validValues = Array('p.published','p.title','jc.title','p.votes');
        if(!in_array($filter_order, $validValues)) {
            $filter_order = 'p.published';
            $filter_order_Dir = 'DESC';
        }
        $orderby            = ' ORDER BY '. $filter_order .' '. $filter_order_Dir;
        $where = '';
        if (empty($this->_data)) {

            $eligibility = '';
            if($user && $user->id > 0) {
                $eligibility = ', (SELECT COUNT(*) FROM ' . TABLE_JCP_VOTES .
                        ' WHERE poll_id=p.id AND voter_id=' . $user->id . ') AS eligible';
            }else {
                $eligibility = ', 1 AS eligible';
            }
            $where .= ($userids || $userids == '0')?
                    ' WHERE p.created_by IN (' . $userids . ') AND p.published=1':
                    ' WHERE p.created_by=' . $user->id;
            $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                    ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                    ' p.created as created, p.published, u.username' . $eligibility  . ',' .
                    ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_POLLS .
                    ' p LEFT JOIN #__users u ON p.created_by=u.id ' .
                    ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                    $where . $orderby;

            $this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
        }
        return $this->_data;
    }

    function &getPollsVotedByUser($userids=null) {
        global $option;
        $app = &JFactory::getApplication();
        $user =& JFactory::getUser();

        $filter_order       = $app->getUserStateFromRequest( "$option.filter_order",	'filter_order',		'v.voted_on',	'cmd' );
        $filter_order_Dir   = $app->getUserStateFromRequest( "$option.filter_order_Dir",	'filter_order_Dir',	'DESC', 'word' );
        $validValues = Array('v.voted_on','p.title','jc.title','p.votes');
        if(!in_array($filter_order, $validValues)) {
            $filter_order = 'v.voted_on';
            $filter_order_Dir = 'DESC';
        }
        $orderby            = ' ORDER BY '. $filter_order .' '. $filter_order_Dir;
        $where = '';
        if (empty($this->_data)) {

            $where .= (count($userids))?
                    ' WHERE v.voter_id IN (' . $userids . ') AND p.published=1':
                    ' WHERE v.voter_id = ' . $user->id;
            $query = 'SELECT p.id, p.title, CASE WHEN CHAR_LENGTH(p.alias) THEN' .
                    ' CONCAT_WS(\':\', p.id, p.alias) ELSE p.id END as slug, p.votes,' .
                    ' p.created as created, p.published, u.username, v.voted_on,' .
                    ' jc.id AS catid, jc.title AS category FROM ' . TABLE_JCP_VOTES . ' v' .
                    ' LEFT JOIN ' . TABLE_JCP_POLLS . ' p ON v.poll_id=p.id' .
                    ' LEFT JOIN ' . TABLE_JCP_CATEGORIES . ' jc ON p.category=jc.id' .
                    ' LEFT JOIN #__users u ON p.created_by=u.id ' .
                    $where . $orderby;

            $this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
        }
        return $this->_data;
    }

    function traceEligibility($poll_id, $skipDB) {
        global $option;
        $app = &JFactory::getApplication();
        $eligible = true;
        if(!$poll_id) {
            return false;
        }

        $cookieName	= JUtility::getHash( $app->getName() . COOKIE_PREFIX . $poll_id );
        if(isset($_COOKIE[$cookieName])) {
            $eligible = false;
        }else if(!$skipDB) {
            $db =& JFactory::getDBO();
            $user =& JFactory::getUser();
            $query = 'SELECT COUNT(*) FROM ' . TABLE_JCP_VOTES . ' WHERE poll_id=' .
                    $poll_id . ' AND voter_id=' . $user->id;
            $db->setQuery($query);
            $count = $db->loadResult();
            if($count > 0) {
                $eligible = false;
            }
        }
        return $eligible;
    }

    function getTotal($userids, $task) {
        if (empty($this->_total)) {
            $user =& JFactory::getUser();
            switch ($task) {
                case 'POLLS':
                    $where = ($userids)?' WHERE created_by IN (' . $userids . ') AND published=1':
                            ' WHERE created_by=' . $user->id;
                    $query = 'SELECT * FROM ' . TABLE_JCP_POLLS . $where;
                    break;
                case 'VOTES':
                    $where = ($userids)?' WHERE voter_id IN (' . $userids . ')':
                            ' WHERE voter_id=' . $user->id;
                    $query = 'SELECT * FROM ' . TABLE_JCP_VOTES . $where;
                    break;
            }
            $this->_total = $this->_getListCount($query);
        }
        return $this->_total;
    }

    function getPagination($userids=null, $task='POLLS') {
        // Load the content if it doesn't already exist
        if (empty($this->_pagination)) {
            jimport('joomla.html.pagination');
            $this->_pagination = new JPagination($this->getTotal($userids,$task), $this->getState('limitstart'), $this->getState('limit') );
        }

        return $this->_pagination;
    }
}
?>