<?php
/**
 * Helper class for leaderboard module
 * 
 * @package    Joomla
 * @subpackage Modules
 * @link http://www.corejoomla.com
 * @license        GPL
 * Displays top users
 */
class modleaderboardHelper
{
    /**
     * Retrieves the hello message
     *
     * @param array $params An object containing the module parameters
     * @access public
     */    
    function &getLeaders( $params ){
        $frequency = intval($params->get('frequency',60));
        
        $lock = JPATH_ROOT . DS . 'modules' . DS . 'mod_leaderboard' . DS . '.LOCK';
    		$file_data = JPATH_ROOT . DS . 'modules' . DS . 'mod_leaderboard' . DS . '.DATA';
        $leaders = array();
    		$end_time = time();
				jimport('joomla.filesystem.file');
        if(JFile::exists($lock)) {
        	$start_time = trim(JFile::read($lock,false,15));
        	$work_time =  (($end_time - $start_time)/60);
        	if($work_time > $frequency){
        		$leaders = &modleaderboardHelper::fetchLeaders($params);
        		$sData = serialize($leaders);
        		if(file_exists($file_data)){
        			JFile::delete($file_data);
        		}
        		JFile::write($file_data, $sData);
        		JFile::write($lock,$end_time);
        	}else{
        		$sData = file_get_contents($file_data);
        		$leaders = unserialize($sData);
        	}
        }else{
        		$leaders = &modleaderboardHelper::fetchLeaders($params);
        		$sData = serialize($leaders);
        		JFile::write($file_data, $sData);
        		JFile::write($lock,$end_time);
        }
        
        return $leaders;
    }
    
    function &fetchLeaders($params){
      $count = intval($params->get('count',5));
      $ruleids = trim($params->get('ruleids',null));
      $excludeusers = trim($params->get('excludeusers',null));
      $mweekly = intval($params->get('mweekly','1'));
      $showlast = intval($params->get('showlast',0));
      $points_system = trim($params->get('points_system','jomsocial'));
      switch($points_system){
      	case 'jomsocial':
    			return modleaderboardHelper::fetchLeadersJS($count, $ruleids, $mweekly, $showlast, $excludeusers);
    		case 'aup':
    			return modleaderboardHelper::fetchLeadersAUP($count,$ruleids, $mweekly, $showlast, $excludeusers);
    		default:
    			return modleaderboardHelper::fetchLeadersJS($count,$ruleids, $mweekly, $showlast, $excludeusers);
    	}
    }
    
    function &fetchLeadersJS($count=5, $ruleids=null, $mweekly='1', $showlast=0, $excludeusers=null){
    	$db = &JFactory::getDBO();
    	$where[] = "a.actor > 0";
    	if($mweekly == '1'){//weekly
    		$where[] = "a.created >= SUBDATE(CURDATE(), INTERVAL WEEKDAY(NOW()) DAY)";
    	}else{//monthly
    		$where[] = "a.created >= DATE_FORMAT(NOW(), '%Y-%m-01')";
    	}
    	if($excludeusers){
    		$where[] = "a.actor NOT IN (" . $excludeusers . ")";;
    	}
    	$where = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
    	
    	$query = "SELECT u.id,u.name,u.username,sum(a.points) AS points FROM #__community_activities a 
    		LEFT JOIN #__users u ON a.actor=u.id".$where." GROUP BY u.id ORDER BY points DESC";
    	$db->setQuery($query,0,$count);
    	$current=$db->loadObjectList();
    	$result->current = $current;
    	$where = null;
    	if($showlast){
    		$where = array();
	    	$where[] = "a.actor > 0";
	    	if($mweekly == '1'){//weekly
	    		$where[] = "a.created >= SUBDATE(CURDATE(), INTERVAL 7+WEEKDAY(NOW()) DAY) AND a.created < SUBDATE(CURDATE(), INTERVAL WEEKDAY(NOW()) DAY)";
	    	}else{//monthly
	    		$where[] = "a.created >= DATE_SUB(DATE_FORMAT(NOW(), '%Y-%m-01'), INTERVAL 1 MONTH) AND  a.created < DATE_FORMAT(NOW(), '%Y-%m-01')";
	    	}
	    	if($excludeusers){
	    		$where[] = "a.actor NOT IN (" . $excludeusers . ")";;
	    	}
	    	$where = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
	    	
	    	$query = "SELECT u.id,u.name,u.username,sum(a.points) AS points FROM #__community_activities a 
	    		LEFT JOIN #__users u ON a.actor=u.id".$where." GROUP BY u.id ORDER BY points DESC";
	    	$db->setQuery($query,0,$count);
	    	$last=$db->loadObjectList();
	    	$result->last = $last;
    	}
    	return $result;
    }
    
    function &fetchLeadersAUP($count=5,$ruleids=null, $mweekly='1', $showlast=0, $excludeusers=null){
    	$db = &JFactory::getDBO();
    	$where[] = "au.userid > 0";
    	if($ruleids){
    		$where[] = "aud.rule NOT IN (".$ruleids.")";
    	}
    	if($mweekly == '1'){//weekly
    		$where[] = "aud.insert_date >= SUBDATE(CURDATE(), INTERVAL WEEKDAY(NOW()) DAY)";
    	}else{//monthly
    		$where[] = "aud.insert_date >= DATE_FORMAT(NOW(), '%Y-%m-01')";
    	}
    	if($excludeusers){
    		$where[] = "au.userid NOT IN (" . $excludeusers . ")";;
    	}
    	$where = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
    	
    	$query = "SELECT u.id,u.name,u.username,sum(aud.points) AS points FROM #__alpha_userpoints au 
    		LEFT JOIN #__alpha_userpoints_details aud ON au.referreid=aud.referreid 
    		LEFT JOIN #__users u ON au.userid=u.id".$where." GROUP BY u.id ORDER BY points DESC";
    	$db->setQuery($query,0,$count);
    	$current=$db->loadObjectList();
    	$result->current = $current;
    	$where = null;
    	if($showlast){
    		$where = array();
	    	$where[] = "au.userid > 0";
	    	if($ruleids){
	    		$where[] = "aud.rule NOT IN (".$ruleids.")";
	    	}
	    	if($mweekly == '1'){//weekly
	    		$where[] = "aud.insert_date >= SUBDATE(CURDATE(), INTERVAL 7+WEEKDAY(NOW()) DAY) AND aud.insert_date < SUBDATE(CURDATE(), INTERVAL WEEKDAY(NOW()) DAY)";
	    	}else{//monthly
	    		$where[] = "aud.insert_date >= DATE_SUB(DATE_FORMAT(NOW(), '%Y-%m-01'), INTERVAL 1 MONTH) AND  aud.insert_date < DATE_FORMAT(NOW(), '%Y-%m-01')";
	    	}
	    	if($excludeusers){
	    		$where[] = "au.userid NOT IN (" . $excludeusers . ")";;
	    	}
	    	$where = ( count( $where ) ? ' WHERE ' . implode( ' AND ', $where ) : '' );
	    	
	    	$query = "SELECT u.id,u.name,u.username,sum(aud.points) AS points FROM #__alpha_userpoints au 
	    		LEFT JOIN #__alpha_userpoints_details aud ON au.referreid=aud.referreid 
	    		LEFT JOIN #__users u ON au.userid=u.id".$where." GROUP BY u.id ORDER BY points DESC";
	    	$db->setQuery($query,0,$count);
	    	$last=$db->loadObjectList();
	    	$result->last = $last;
    	}
    	return $result;
    }
    
    function getUserAvtar($avatarType='jomsocial', $userid=0, $height=48, $username='username') {
        $db = JFactory::getDBO();
        switch ( $avatarType ) {
            case 'jomsocial':
                $jspath = JPATH_BASE.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php';
                if(file_exists($jspath)){
	                include_once($jspath);
	
	                // Get CUser object
	                $jsuser =& CFactory::getUser($userid);
	                $usrname = $jsuser->$username;
	                $avatarLoc = $jsuser->getThumbAvatar();
	                $link = CRoute::_('index.php? option=com_community&view=profile&userid='.$userid.'&Itemid='.JRequest::getInt('Itemid'));
	                $avatar = "<a href=\"$link\"><img class=\"hasTip\"
	                        style=\"border: 1px solid #cccccc; height: " . $height . "px;\"
	                        src=\"$avatarLoc\" alt=\"$usrname\"  title=\"$usrname\" /></a>";
                }
                break;
            case 'cb':
                $strSql = "SELECT `avatar`, firstname FROM #__comprofiler WHERE `user_id`='{$userid}' AND `avatarapproved`='1'";
                $db->setQuery($strSql);
                $result = $db->loadObject();
                $link = JRoute::_( 'index.php?option=com_comprofiler&amp;task=userProfile&amp;user=' . $userid);
                if($result && !empty($result->avatar)) {
                    $avatarLoc = JURI::base(true)."/images/comprofiler/".$result->avatar;
                } else {
                    $avatarLoc = JURI::base(true)."/components/com_comprofiler/plugin/templates/default/images/avatar/nophoto_n.png";
                }
                $avatar = "<a href=\"" . $link . "\"><img src=\"" . $avatarLoc . "\"" .
                        "class=\"hasTip\" style=\"border: 1px solid #cccccc; height: " . $height . "px;\"" .
                        "alt=\"" . $result->firstname . "\" title=\"" . $result->firstname . "\"/></a>";
                break;
            case 'touch':
                $avatarLoc = JURI::base(true) . '/index2.php?option=com_community&amp;controller=profile&amp;task=avatar&amp;width=' . $height . '&amp;height=' . $height . '&amp;user_id=' . $userid . '&amp;no_ajax=1';
                $avatar = '<img src="' . $avatarLoc . '" style="border: 1px solid #cccccc; height: "' . $height . '"px;" alt=""/>';
                $link = JRoute::_("index.php?option=com_community&view=profile&user_id={$userid}");
                $avatar = '<a href="' . $link . '">' . $avatar . '</a>';
                break;
            case 'kunena':
                $query = "SELECT a.avatar, b." . $username . " FROM #__fb_users as a" .
                        " LEFT JOIN #__users as b on b.id=a.userid where a.userid=".$userid;
                $db->setQuery( $query );
                $kunenaUser = $db->loadObject();
                $kunena_avtar = @$kunenaUser->avatar;
                $kunena_avtars = JURI::base(true) . '/images/fbfiles';
                if ($kunena_avtar != '') {
                    if(!file_exists( JPATH_ROOT .'/images/fbfiles/avatars/l_' . $kunena_avtar)) {
                        $avatarLoc = $kunena_avtars . '/avatars/' . $kunena_avtar;
                    }
                    else {
                        $avatarLoc = $kunena_avtars . '/avatars/l_' . $kunena_avtar;
                    }
                }
                else $avatarLoc = $kunena_avtars . '/avatars/nophoto.jpg';
                //URL
                $usrname = ($kunenaUser->$username)?$kunenaUser->$username:JText::_('GUEST');
                $link = JRoute::_( 'index.php?option=com_kunena&amp;func=fbprofile&amp;userid=' . $userid);
                $avatar = "<a href=\"" . $link . "\" title=\"" . $usrname . "\"><img src=\"" . $avatarLoc . "\"" .
                        "class=\"hasTip\" style=\"border: 1px solid #cccccc; height: " . $height . "px;\"" .
                        "alt=\"" . $usrname . "\"/></a>";
                break;
            case 'aup':
                $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                if ( file_exists($api_AUP)){
                    require_once ($api_AUP);
                    $avatar = AlphaUserPointsHelper::getAupAvatar($userid, 1, $height, $height);
                }
                break;
        }
        return $avatar;
    }
    
    function getUserProfileUrl($avatar='jomsocial', $userid=null, $username='Guest') {
        $db = JFactory::getDBO();
        $link = null;
        switch ( $avatar ) {
            case 'jomsocial':
                $jspath = JPATH_BASE.DS.'components'.DS.'com_community'.DS.'libraries'.DS.'core.php';
                if(file_exists($jspath)) {
                    include_once($jspath);
                    $link = '<a href="' . CRoute::_('index.php? option=com_community&view=profile&userid='.$userid) . '">' . $username . '</a>';
                }
                break;
            case 'cb':
                $link = '<a href="' . JRoute::_( 'index.php?option=com_comprofiler&amp;task=userProfile&amp;user=' . $userid) . '">' . $username . '</a>';
                break;
            case 'touch':
                $link = modleaderboardHelper::getTouchPopup($userid, $username);
                break;
            case 'kunena':
                $link = '<a href="' . JRoute::_( 'index.php?option=com_kunena&amp;func=fbprofile&amp;userid=' . $userid) . '">' . $username . '</a>';
                break;
            case 'aup':
                $api_AUP = JPATH_SITE.DS.'components'.DS.'com_alphauserpoints'.DS.'helper.php';
                if ( file_exists($api_AUP)){
                    require_once ($api_AUP);
                    $link = AlphaUserPointsHelper::getAupLinkToProfil($userid);
                }
                break;
            default:
                return null;
        }
        return (!$link)?$username:$link;;
    }
    
    function getTouchPopup($user_id, $user_name) {
        Static $capi = false;
        Static $api_enabled = true;

        if($api_enabled == false) return $user_name;

        if(!$capi) {
            $API = JPATH_ROOT.DS.'components'.DS.'com_community'.DS.'api.php';
            if(file_exists($API)) {
                require_once $API;
                $capi = new JSCommunityApi();
            }
            else {
                $api_enabled = false;
                return $user_name;
            }
        }

        $params['width'] = 400;
		$links = array();
//        $links[0]->link = JRoute::_( 'index.php?option=com_communitypolls&controller=polls&task=userpolls&uid='.$user_id.'&Itemid='.JRequest::getInt('Itemid'));
//        $links[0]->alt = JText::_('LBL_ALL_POLLS_FROM') . ' this user';
//        $links[0]->icon = 'components/com_communitypolls/assets/images/poll_add.png';

        $user_name = $capi->getUserSlideMenu($user_id, $user_name, $links, $params);

        return $user_name;
    }
    
    function writeLeadersData($params,$leader){
			$avatarComponent = trim($params->get('avatar','none'));
			$username = trim($params->get('username','name'));
			$avatarSize = trim($params->get('avatar_size','24'));
			$avatar = modleaderboardHelper::getUserAvtar($avatarComponent, $leader->id, $avatarSize, $username);
			$profile = modleaderboardHelper::getUserProfileUrl($avatarComponent, $leader->id, $leader->$username);
			echo '<div class="leaderItem">';
			if(!empty($avatar)){
				echo '<div class="leaderAvatar">' . $avatar . '</div>';
			}
			echo '<div class="leaderName">' . $profile . '</div>';
			echo '<div class="leaderPoints"><strong>Points: </strong>' . $leader->points . '</div>';
			echo '</div>';
			echo '<div style="clear: both;"></div>';
    }
    
    function getFrequncyText($minutes){
    	$hours = 0;
    	if($minutes >= 60){
    		$hours = round($minutes/60);
    	}
			$minutes = $minutes % 60;
			return JText::_('UPDATED_EVERY') . (($hours > 0)?' '.$hours.' '.JText::_('HOURS'):'') . (($minutes>0)?' '.$minutes.' '.JText::_('MINUTES'):'');
    }
}
?>
