<?php
/**
 * @version             $Id: communitypolls.php v1.0.0 2/9/2010 Maverick
 * @copyright           Copyright 2009-10 corejoomla.com
 * @license             License, for example GNU/GPL
 * Search plugin for community answers extension.
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

$app = &JFactory::getApplication();

//Now define the registerEvent and the language file. 
$app->registerEvent( 'onSearch', 'plgSearchCommunityPolls' );
$app->registerEvent( 'onSearchAreas', 'plgSearchCommunityPollsAreas' );

JPlugin::loadLanguage( 'plg_search_communitypolls' );

/**
 * @return array An array of search areas
 */
function &plgSearchCommunityPollsAreas()
{
	static $areas = array(
		'CommunityPolls' => 'Polls'
		);
		return $areas;
}

/**
 * Community Polls Search method
 *
 * The sql must return the following fields that are
 * used in a common display routine: href, title, section, created, text, browsernav
 * @param string Target search string
 * @param string mathcing option, exact|any|all
 * @param string ordering option, newest|oldest|popular|alpha|category
 * @param mixed An array if restricted to areas, null if search all
 */
function plgSearchCommunityPolls( $text, $phrase='', $ordering='', $areas=null )
{
	$db		=& JFactory::getDBO();
	$user	=& JFactory::getUser();
	$searchText = $text;

	require_once(JPATH_SITE.DS.'components'.DS.'com_communitypolls'.DS.'router.php');

	//Then the real function has to be created. The database connection should be made.
	if (is_array( $areas )) {
		if (!array_intersect( $areas, array_keys( plgSearchCommunityPollsAreas() ) )) {
			return array();
		}
	}
	//It is time to define the parameters! First get the right plugin; 'search' (the group), 'communitypolls'.
	$plugin =& JPluginHelper::getPlugin('search', 'communitypolls');

	//Then load the parameters of the plugin..
	$pluginParams = new JParameter( $plugin->params );

	//And define the parameters. For example like this..
	$limit = $pluginParams->def( 'search_limit', 50 );

	//Use the function trim to delete spaces in front of or at the back of the searching terms
	$text = trim( $text );

	//Return Array when nothing was filled in
	if ($text == '') {
		return array();
	}

	//After this, you have to add the database part. This will be the most difficult part, because this changes per situation.
	$wheres = array();
	switch ($phrase) {

		//search exact
		case 'exact':
			$text			= $db->Quote( '%'.$db->getEscaped( $text, true ).'%', false );
			$wheres2		= array();
			$wheres2[]		= 'LOWER(a.title) LIKE '.$text;
			$wheres2[]		= 'LOWER(a.description) LIKE '.$text;
			$where			= '(' . implode( ') OR (', $wheres2 ) . ')';
			break;

			//search all or any
		case 'all':
		case 'any':
		default:
			$words         = explode( ' ', $text );
			$wheres = array();
			foreach ($words as $word)
			{
				$word        = $db->Quote( '%'.$db->getEscaped( $word, true ).'%', false );
				$wheres2     = array();
				$wheres2[]   = 'LOWER(a.title) LIKE '.$word;
				$wheres2[]   = 'LOWER(a.description) LIKE '.$word;
				$wheres[]    = implode( ' OR ', $wheres2 );
			}
			$where = '(' . implode( ($phrase == 'all' ? ') AND (' : ') OR ('), $wheres ) . ')';
			break;
	}

	//ordering of the results
	switch ( $ordering ) {

		//alphabetic, ascending
		case 'alpha':
			$order = 'a.title ASC';
			break;

			//oldest first
		case 'oldest':
			$order = 'a.created ASC';
			break;
			
			//popular first
		case 'popular':
			$order = 'a.votes DESC';
			break;
			
			//newest first
		case 'newest':
			$order = 'a.created DESC';
			break;
			
			//default setting: alphabetic, ascending
		default:
			$order = 'a.title ASC';
	}

	//replace community answers
	$searchNameofplugin = JText::_( 'CommunityPolls' );

	//the database query; differs per situation! It will look something like this:
	$query = 'SELECT a.id, a.alias, a.title AS title,'
	. ' CONCAT_WS( " / ", '. $db->Quote($searchNameofplugin) .', c.title ) AS section,'
	. ' "1" AS browsernav'
	. ' FROM #__jcp_polls AS a'
	. ' LEFT JOIN #__jcp_categories AS c ON c.id = a.category'
	. ' WHERE ( '. $where .' )'
	. ' AND a.published = 1'
	. ' ORDER BY '. $order
	;
	//Set query
	$db->setQuery( $query, 0, $limit );
	$rows = $db->loadObjectList();
	
	//The 'output' of the displayed link
	if($rows){
		foreach($rows as $key => $row) {
			$rows[$key]->href = 'index.php?option=com_communitypolls&controller=polls&task=viewpoll&id='.$row->id.':'.$row->alias;
		}
	}

	//Return the search results in an array
	return $rows;
}
